/*********************************************************************
* Advanced search bar with built in voice recognition and real-time
* Wikipedia API integration. Toggle between light and dark themes, 
* with sleek, modern animations, and appearances. Tested on Chrome,
* Edge, Opera, and Safari browsers, and ready to go live.
* Note: Firefox does not support Speech or Wikipedia APIs.
* Written by Chad Jordan © interactivenexus.com
* 5/17/2018
*********************************************************************/

const searchBarContainerEl = document.querySelector(".search-bar-container");
const magnifierEl = document.querySelector(".magnifier");
const inputEl = document.querySelector(".input");
const micIconEl = document.querySelector(".mic-icon");

magnifierEl.addEventListener("click", () => {
  searchBarContainerEl.classList.toggle("active");
});

const body = document.body;
const toggleDarkMode = document.createElement("button");
toggleDarkMode.id = "toggleDarkMode";
toggleDarkMode.innerText = "🌙";
toggleDarkMode.style.position = "absolute";
toggleDarkMode.style.top = "10px";
toggleDarkMode.style.right = "10px";
document.body.appendChild(toggleDarkMode);

toggleDarkMode.addEventListener("click", () => {
  document.body.classList.toggle("dark-mode");
  //Dynamically change icon
  if (document.body.classList.contains("dark-mode")) {
    toggleDarkMode.innerText = "☀️";
  } else {
    toggleDarkMode.innerText = "🌙";
  }
});

const searchInput = document.querySelector(".input");
const searchHistory = JSON.parse(localStorage.getItem("searchHistory")) || [];

searchInput.addEventListener("keyup", (event) => {
  if (event.key === "Enter") {
    const query = searchInput.value.trim();
    if (query && !searchHistory.includes(query)) {
      searchHistory.push(query);
      localStorage.setItem("searchHistory", JSON.stringify(searchHistory));
    }
  }
});

const micIcon = document.querySelector(".mic-icon");
const recognition = new (window.SpeechRecognition || window.webkitSpeechRecognition)();

micIcon.addEventListener("click", () => {
  recognition.start();
});

function performWikiSearch(searchText) {
  const query = searchText.trim();
  if (query) {
      fetch(`https://en.wikipedia.org/w/api.php?action=opensearch&format=json&search=${encodeURIComponent(query)}&limit=5&namespace=0&origin=*`)
          .then(response => response.json())
          .then(data => {
              searchResultsContainer.style.display = "block";
              searchResultsContainer.innerHTML = data[1]
                  .map(title => `
                      <div class="search-item" onclick="window.open('https://en.wikipedia.org/wiki/${encodeURIComponent(title)}', '_blank')">
                          ${title}
                      </div>
                  `).join("");
          });
  }
}

recognition.onresult = (event) => {
  const transcript = event.results[0][0].transcript;
  inputEl.value = transcript;
  performWikiSearch(transcript);
};

inputEl.addEventListener("input", () => {
  performWikiSearch(inputEl.value);
});

inputEl.addEventListener("focus", () => {
  searchBarContainerEl.classList.remove("active"); 
});

inputEl.addEventListener("blur", () => {
  if (!inputEl.value) {
      searchBarContainerEl.classList.add("active");
  }
});

//Check if the browser supports Speech Recognition
const SpeechRecognition = window.SpeechRecognition || window.webkitSpeechRecognition;
if (SpeechRecognition) {
  const recognition = new SpeechRecognition();
  recognition.continuous = false; //Stops listening after the user finishes speaking
  recognition.interimResults = false; 
  recognition.lang = "en-US";
  recognition.maxAlternatives = 1;

//Extend recording time by resetting timeout
let recognitionTimeout;
recognition.addEventListener("start", () => {
    clearTimeout(recognitionTimeout); 
    recognitionTimeout = setTimeout(() => {
      searchBarContainerEl.classList.remove("listening");
      inputEl.placeholder = "Search...";
      recognition.stop();
    }, 10000); //10 second timer (default + 5 sec)
});

  micIconEl.addEventListener("click", () => {
    searchBarContainerEl.classList.add("listening");
    inputEl.placeholder = "Listening..."; 
    recognition.start();
});

  //Insert spoken text into input field and keep expanded if text exists
  recognition.onresult = (event) => {
    const transcript = event.results[0][0].transcript;
    inputEl.value = transcript; 
    searchBarContainerEl.classList.remove("active"); 
  };

  recognition.onspeechend = () => {
    searchBarContainerEl.classList.remove("listening");
    inputEl.placeholder = "Search...";
    recognition.stop();
  };

  recognition.onerror = (event) => {
    console.log("Voice recognition error:", event.error);
  };
} else {
  console.log("Web Speech API is not supported in this browser.");
}

function updateGlowMode() {
  if (window.matchMedia && window.matchMedia("(prefers-color-scheme: dark)").matches) {
    document.documentElement.style.setProperty("--glow-color", "rgba(255, 0, 0, 1)");
  } else {
    document.documentElement.style.setProperty("--glow-color", "rgba(255, 0, 0, 0.8)");
  }
}

updateGlowMode();

//Listener for theme changes
window.matchMedia("(prefers-color-scheme: dark)").addEventListener("change", updateGlowMode);

const updateSearchResultsTheme = () => {
  const searchResults = document.querySelector(".search-results");
  if (searchResults) {
    if (document.body.classList.contains("dark-mode")) {
      searchResults.classList.add("dark-mode");
    } else {
      searchResults.classList.remove("dark-mode");
    }
  }
};
toggleDarkMode.addEventListener("click", updateSearchResultsTheme);

//Handle Wikipedia API calls 
let debouncerTimer;
async function getWikipediaSuggestions(query) {
    const endpoint = `https://en.wikipedia.org/w/api.php?action=opensearch&format=json&search=${encodeURIComponent(query)}&limit=5&namespace=0&origin=*`;
    
    try {
        const response = await fetch(endpoint);
        const data = await response.json();
        return data[1]; //Returns an array of suggested titles
    } catch (error) {
        console.error('Error fetching Wikipedia suggestions:', error);
        return [];
    }
}

//Fetch results from Wikipedia in real-time as you type
const searchResultsContainer = document.createElement("div");
searchResultsContainer.classList.add("search-results");
searchBarContainerEl.appendChild(searchResultsContainer);

searchInput.addEventListener("input", async () => {
  const query = searchInput.value.trim();
  
  if (query === "") {
      searchResultsContainer.innerHTML = "";
      searchResultsContainer.style.display = "none";
      return;
  }

  const endpoint = `https://en.wikipedia.org/w/api.php?action=opensearch&format=json&search=${encodeURIComponent(query)}&limit=5&namespace=0&origin=*`;
  
  try {
      const response = await fetch(endpoint);
      const data = await response.json();
      const suggestions = data[1];
      
      searchResultsContainer.style.display = "block";
      searchResultsContainer.innerHTML = suggestions
          .map(title => `
              <div class="search-item" onclick="window.open('https://en.wikipedia.org/wiki/${encodeURIComponent(title)}', '_blank')">
                  ${title}
              </div>
          `).join("");
  } catch (error) {
      console.error('Error fetching Wikipedia suggestions:', error);
  }
});
